;***********************************************************************
;*
;* File Name		:"PS2.asm"
;* Title		:PS/2 communication protocoll
;* 
;* Date (Orig.)		:2001.10.06.
;* Date (Last)		:2001.10.12.
;* Version 		:1.0.10
;* Support telephone	:+36-30-9541-658
;* Support fax		:+36-30-9809-372
;* Support E-mail	:info@vfx.hu
;* Target MCU		:AT90S8515
;*
;* DESCRIPTION
;*  - PS/2 style communication to AT or MARGOR CP-01
;*
;*	CLK & DATA = H	-> Two way communication 
;*	DATA = L	-> PC (MAGOR) sends to Keyb
;*	CLK = L		-> PC (MAGOR) inhibits communiaction

;**************************************************************************
;* Const Def


;***************************************************************************
;***** Hardware Abstraction Layer

.EQU	PS2_CLKPort = DDRB
.EQU	PS2_DATPort = DDRB

.EQU	KCLK  = 0
.EQU	KDATA = 1


;******************************************************************************
;**** VARIABLES
.DSEG

.EQU	NoError   = 0		;No Error
.EQU	RxF_Abort = 2		;Abort communication
.EQU	TxBreak   = 3		;Abort Transmit
.EQU	Parity_Err= 4		;Parity Error


;******************************************************************************
.ESEG


;******************************************************************************
.CSEG

Init_PS2:	CBI	PS2_CLKPort,KCLK
         	CBI	PS2_CLKPort+1,KCLK	;CLK pin tre-stated external pull-up res!!

		CBI	PS2_DATPort,KDATA
         	CBI	PS2_DATPort+1,KDATA	;DATA pin tre-stated external pull-up res!!

		clr	R16
		STD	Y+PS2NextA+0-RAMSTRT,R16
		STD	Y+PS2NextA+1-RAMSTRT,R16 ;nincs aktiv komunikacio folyamatban
		STD	Y+PS2Flag-RAMSTRT,R16 

		ret


;******************************************************************************
;** Communication between Host and Keyboard
;**
;******************************************************************************
PS2_Com:	sbis	PS2_CLKPort-1,KCLK
		 ret				;if clk pin low, comm. inhibit
		sbis	PS2_DATPort-1,KDATA
		 rjmp	RxCommand		;if data pin low, HOST sends command/data


		LDD	R16,Y+PS2Flag-RAMSTRT	;Request Resend?
		andi	R16,0b00000100
		breq	Ps2_C2
		ldi	R16,0xFE		;RESEND Command
		mov	R1,R16
		rcall	SendByte		;keres elkuldese
		brcs	ResSendERR
		LDD	R16,Y+PS2Flag-RAMSTRT	;Request Resend flag torlese
		andi	R16,0b11111011
		STD	Y+PS2Flag-RAMSTRT,R16
		ret


Ps2_C2:		LDD	R16,Y+PS2Flag-RAMSTRT	;Resend last data?
		andi	R16,0b00000010
		breq	Ps2_C0
		ldd	R1,Y+LastSend-RAMSTRT	;utoljara kuldott karakter ismetleshez
		rcall	SendByte		;ujrakuldes
		brcs	ResSendERR
		LDD	R16,Y+PS2Flag-RAMSTRT	;Resend flag torlese
		andi	R16,0b11111101
		STD	Y+PS2Flag-RAMSTRT,R16
ResSendERR:	ret				;ha hiba volt ujra kuldjuk

Ps2_C0:		LDD	R16,Y+PS2Flag-RAMSTRT	;Send ACK flag active?
		andi	R16,0b00000001
		breq	Ps2_C1
		ldi	R16,0xFA		;Send ACK to HOST
		mov	R1,R16
		rcall	SendByte
		brcs	ResSendERR		;ha hiba van ujrakuldes lesz
		LDD	R16,Y+PS2Flag-RAMSTRT	;Host Abort
		andi	R16,0b11111110		;ACK flag torlove sikeresen elment		
		STD	Y+PS2Flag-RAMSTRT,R16	;komunikacio nincs
		ret

Ps2_C1:		LDD	R16,Y+PS2Flag-RAMSTRT	;Parameterre var?
		andi	R16,0b00001000
		breq	Ps2_C3			;ha nem, akkor hadmenjen

		LDD	R16,Y+ParamErr-RAMSTRT	;parameterr error counter>4ms -> igen, hiba
                inc	R16
		cpi	R16,20			;20*250us ido ~4..5ms
		brcs	Megidovan		;idotullepes!!

		LDD	R16,Y+PS2Flag-RAMSTRT	;Parameterre var bit torlese, nem varunk tovabb!
		andi	R16,0b11110111
		STD	Y+PS2Flag-RAMSTRT,R16
		clr	R16
		STD	Y+PS2NextA+0-RAMSTRT,R16
		STD	Y+PS2NextA+1-RAMSTRT,R16 ;alapallapotba hozas!
Megidovan:	STD	Y+ParamErr-RAMSTRT,R16
		ret


Ps2_C3:		LDD	ZL,Y+PS2NextA+0-RAMSTRT
		LDD	ZH,Y+PS2NextA+1-RAMSTRT
		mov	R16,ZL
		or	R16,ZH			;ha nincs komunikacio folyamatban
		breq	SendScan		;akkor kuldjunk scan kodot
		ijmp				;ugras a komunikacios rutinra

;********************************************
;** Send scan/fifo
;********************************************
SendScan:	ldi	ZL,Low(KeyBuff)
		ldi	ZH,High(KeyBuff)
		LDD	R0,Y+KeyHead-RAMSTRT
		LDD	R16,Y+KeyTrail-RAMSTRT
		cp	R0,R16
		brne	SendScn1
		ret
SendScn1:	clr	R0
		add	Zl,R16
		adc	ZH,R0
		ld	R1,Z
		inc	R16
		cpi	R16,KeyBuffLen
		brcs	SendScn2
		clr	R16
SendScn2:	STD	Y+KeyTrail-RAMSTRT,R16	;uj pointer elmentve
		rcall	SendByte
		brcc	NoSendErr		;nincs hiba siman elment
		LDD	R16,Y+PS2Flag-RAMSTRT	;Resend flag beallitasa
		ori	R16,0b00000010
		STD	Y+PS2Flag-RAMSTRT,R16
NoSendErr:	ret


;*****************************************
;** Send R1 to Host
;** C=1 hiba tortent R17= hibakod
;*****************************************
SendByte:	sbis	PS2_CLKPort-1,KCLK
		 rjmp	SendByte		;wait for clk=hi
		rcall	Delay45us
		rcall	Delay5us		;wait 50us
		sbis	PS2_CLKPort-1,KCLK
		 rjmp	SendByte		;wait for clk=hi
		sec
		std	Y+LastSend-RAMSTRT,R1	;utoljara kuldott karakter
		ldi	R17,TxBreak		;R17= Transmit break
		sbis	PS2_DATPort-1,KDATA	;Is Data still hi?
		 ret				;ha nem,vissza R17 hibakod!
		rcall	Delay40us
		clr	R0			;R0=0 start bit
		rcall	SendBit
		brcs	SendAborted

		ldi	R16,8
		mov	R3,R16
		mov	R0,R1
		clr	R2
SendLoop:	rcall	SendBit
		brcs	SendAborted
		dec	R3
		brne	SendLoop
                inc	R2			;paratlan paritas kell!!!!!???
		mov	R0,R2
		rcall	SendBit
		clr	R0
		dec	R0			;R0=255 stop bit =H
		rcall	SendBit
		rcall	Delay15us
		rcall	Delay15us
		ldi	R17,NoError
                clc
		ret
SendAborted:	ldi	R17,TxBreak		;R17= Transmit break
		sec
		ret		

;* C=1 break, C=0 normal transmit
SendBit:	ror	R0			;0. bit -> C
		brcc	bit0
bit1:         	CBI	PS2_DATPort,KDATA	;DATA pin H
		inc	R2			;paritas no egyel
		rjmp	bitn
bit0:          	SBI	PS2_DATPort,KDATA	;DATA pin L
bitn:		rcall	Delay20us
		SBI	PS2_CLKPort,KCLK	;clk = L
		rcall	Delay40us
		CBI	PS2_CLKPort,KCLK	;clk = H
		rcall	Delay20us
		clc
		sbis	PS2_CLKPort-1,KCLK	;Clock=high?
		 sec				;if no -> C=1
		ret


;******************************************************************************
;* R E C E I V E 
;*
;* 80 us/bit device generated clock - host changes the Data line only when
;* the Clock line is low, and data is latched on the rising edge of the clock pulse
;* Host may abort transmission at time before the 11th clock pulse (acknowledge bit)
;* by holding Clock low for at least 100 microseconds. 
;*
;* Uses:R1,R2,R17,R18

RxStream:	sbis	PS2_CLKPort-1,KCLK	;Wait for Clock=high 
		 rjmp	RxStream

		sbic	PS2_DATPort-1,KDATA	;Is Data still low?
		 rjmp	RxErr			;ha nem, akkor hiba!

;Read each bit (8 data bits, parity bit, and stop bit) as follows: 
		clr	R1			;data reg.
		clr	R2			;parity reg.
		ldi	R17,8			;8 bit

RxBitLoop:	rcall	RxBit			;R18-ban a bit
		add	R2,R18
		ror	R18			;adatbit a C flagben
		ror	R1
		sbis	PS2_CLKPort-1,KCLK	;Clock=high?
		 rjmp	RxAbort			;if no then Abort
		dec	R17
		brne	RxBitLoop

						;Read parity bit 
		rcall	RxBit			;R18-ban a bit
		inc	R2			;paratlan paritas kell!!??!!!?????
                eor	R2,R18			;R2 = parity xor parity bit
		sbis	PS2_CLKPort-1,KCLK
		 rjmp	RxAbort
		
                                          	;Read stop bit 
		rcall	RxBit			;R18-ban a bit
		sbis	PS2_CLKPort-1,KCLK
		 rjmp	RxAbort

		sbis	PS2_DATPort-1,KDATA	;Is Data still low?
		 rjmp	RxErr1			;Data still equals 0?
						;Output Acknowledge bit 
		rcall	SendAckBit		;Acknowledge bit 
		rcall	Delay45us		;Delay 45 microseconds
						;(to give host time to inhibit
						;next transmission.) 
		ldi	R17,0			;no error flag
		ror	R2			;C-flagben a parity
		brcs	RxParErr
		ret				;R1 = reseived data


RxParErr:	ldi	R17,Parity_Err		;Parity Error
		sec				;C flag =0
		ret

RxAbort:	ldi	R17,RxF_Abort		;Abort
		sec				;C flag =0
		ret

;*Keep clocking until Data=1 then generate an error

RxErr1:		rcall	RxBit
		sbis	PS2_DATPort-1,KDATA	;Is Data still low?
		 rjmp	RxErr1			;ha nem, akkor hiba!						;Data still equals 0? Keep clocking until Data=1 then generate an error
RxErr:		
		ret

;Read a bit
RxBit:		rcall	Delay20us
		SBI	PS2_CLKPort,KCLK	;clk = L
		rcall	Delay40us
		CBI	PS2_CLKPort,KCLK	;clk = H
		rcall	Delay20us
		clr	R18			
		sbic	PS2_DATPort-1,KDATA
		 ldi	R18,1			;R18 a beolvasott adatbit
		ret

;Send the acknowledge bit
SendAckBit:	rcall	Delay15us
		SBI	PS2_DATPort,KDATA	;data = L
		rcall	Delay5us
		SBI	PS2_CLKPort,KCLK	;clk = L
		rcall	Delay40us
		CBI	PS2_CLKPort,KCLK	;clk = H
		rcall	Delay5us
		CBI	PS2_DATPort,KDATA	;data = H
		ret



;******************************************************************************
;* Delayxxus
;*
;*rcall [3] + wait [xx] +ret [4] = Delay
;*	@7.3728MHz
;*	45 us [332cycl 
;*	40 us [296cycl 
;*	20 us [148cycl
;*	10 us [74cycl]
;*	 5 us [37cycl]
Delay45us:	ldi	R16,104		;[1] 104*3+10 cycl
		rjmp	DelayLoop	;[2]
Delay5us:	ldi	R16,9		;[1] 9*3+10 cycl
		rjmp	DelayLoop	;[2]
Delay15us:	ldi	R16,34		;[1] 34*3+10 cycl
		rjmp	DelayLoop	;[2]
Delay20us:	ldi	R16,46		;[1] 46*3+10 cycl
		rjmp	DelayLoop	;[2]
Delay40us:	ldi	R16,96		;[1] 96*3 cycl
DelayLoop:	dec	R16		;[1]
		brne	DelayLoop	;[1/2]
		ret			;[4]



;*******************************************
;** Host request to send... receive data
;**
;*******************************************
RxCommand:	LDD	ZL,Y+PS2NextA+0-RAMSTRT
		LDD	ZH,Y+PS2NextA+1-RAMSTRT
		mov	R16,Zl
		or	R16,ZH
		breq	NewCommand
          	ijmp				;ha nem uj, akkor ugras a vegrehajtasra

NewCommand:	rcall	RxStream		;Read data R1, C=1 hiba, R17 hibakod
		brcc	RxComNoErr

ReqResend:	LDD	R16,Y+PS2Flag-RAMSTRT	;Host read error Resend needed
		ori	R16,0b00000100		
		STD	Y+PS2Flag-RAMSTRT,R16
NoValidComm:   	ret

RxComNoErr:	ldi	ZL,Low(2*ComTab)	;uj parancs jott
		ldi	ZH,High(2*ComTab)       		
RxCommLoop:	lpm
		tst	R0			;ha a tabla vege es nincs parancs kihagyjuk
		breq	NoValidComm
		cp	R0,R1		
		breq	HitComm
		adiw	ZL,2*(ComNex-ComTab)	;kovetkezo parancs
		rjmp	RxCommLoop

HitComm:	adiw	ZL,2			;pointerre mutat
		lpm	
		mov	R1,R0			;low byte
		adiw	ZL,1
		lpm
		mov	ZH,R0			;hi byte
		mov	ZL,R1
		ijmp				;ugras a parancs vegrehajtasara

ComTab:		.dw	0xED,STATUS_IND	;Turns on/off LEDs
ComNex:		.dw	0xEE,ECHO	;Echo
		.dw	0xF0,ALT_SCAN	;Set Scan Code Set
		.dw	0xF3,TYPE_RATE	;Set repeat rate 
		.dw	0xF4,ACKAckt	;Enables keysacnning
		.dw	0xF5,ACKAckt	;Disable keyscanning
		.dw	0xF6,SET_DEF	;Set Default Value
		.dw	0xF7,ACKAckt	;Set all keys - Typematic
		.dw	0xF8,ACKAckt	;Set all keys - Make/Break
		.dw	0xF9,ACKAckt	;Set all keys - Make
		.dw	0xFA,ACKAckt	;Set all keys - Typematic/Make/Break
		.dw	0xFB,KEY_TYPE	;Set ket type - Typematic
		.dw	0xFC,KEY_BREAK	;Set ket type - Make/Break
		.dw	0xFD,KEY_MAKE	;Set ket type - Make
		.dw	0xFE,RESEND	;Resend
		.dw	0xFF,KRES	;Reset Command
		.dw	0xF2,READID	;Read ID Command
		.dw	0x00		;ez az utolso sor!




Read_Data:	rcall	RxStream		;Read data R1, C=1 hiba, R17 hibakod
		brcs	RD_Err
		ret
RD_Err:		LDD	R16,Y+PS2Flag-RAMSTRT	;Host read error Resend needed
		ori	R16,0b00000100		
		STD	Y+PS2Flag-RAMSTRT,R16
		sec
	   	ret


;******************************************************************************
;* C O M M A N D S   from host
;*
;******************************************************************************
;*Set/reset status indicators. A '1' bit turns the indicator ON.
;*	0. Scrollock,LED0 Green
;*	1. Numlock,  LED0 Red
;*	2. Capslock, LED1 Green
;*	3. 	     LED1 Red
;*	4. 	     LED2 Green
;*	5. 	     LED2 Red
;*	6. =1 Flash
;*	7. =0  
;*******************************************************
STATUS_IND:	ldi	ZL,Low(StatInd1)
		ldi	ZH,High(StatInd1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 StatInd1:	rcall	Read_Data
		brcc	SatInd2
		ret
 SatInd2:
		LDD	R16,Y+PS2Flag-RAMSTRT
		andi	R16,0b11110111			;parameter renben megjott, torolve a bit
		ori	R16,0b00000001			;ACK kuldes kell
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack

;*******************************************************
ECHO:		ldi	ZL,Low(Echo1)
		ldi	ZH,High(Echo1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
 		ret
 Echo1:		ldi	R16,0xEE			;Echo valasz a parancsra
		mov	R1,R16
		rcall	SendByte			;Send
		rjmp	EndBack

;*******************************************************

ALT_SCAN:	ldi	ZL,Low(Altscan1)
		ldi	ZH,High(Altscan1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 Altscan1:	rcall	Read_Data
		brcc	Altscan2
		ret
 Altscan2:	LDD	R16,Y+PS2Flag-RAMSTRT
		andi	R16,0b11110111			;parameter renben megjott, torolve a bit
		ori	R16,0b00000001			;ACK kuldes kell
		STD	Y+PS2Flag-RAMSTRT,R16

		mov	R16,R1				;parameter byte
		andi	R16,3				;csak az alo 2 bit kell
		cpi	R16,0
		breq	Lekerdezes
		rjmp	EndBack

Lekerdezes:	ldi	ZL,Low(Scanlek)
		ldi	ZH,High(Scanlek)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		ret	

Scanlek:	ldd	R16,Y+Flag1-RAMSTRT
		andi	R16,0b00000011			;scan mode maskolasa 
		inc	R16				;igy lesz 1,2,3 kodtabla
		mov	R1,R16
		rcall	SendByte			;Send scan tab kod
		rjmp	EndBack

;*******************************************************
TYPE_RATE:	ldi	ZL,Low(TYPE_RATE1)
		ldi	ZH,High(TYPE_RATE1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 TYPE_RATE1:	rcall	Read_Data
		brcc	TR1
		ret					;ha hiba volt: request to resend
 TR1:							;Bit 7 is unimplemented. 
							;Bits 6:5 - Repeat Delay
							;Bits 4:0 - Repeat Rate
ACKAckt:
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK flag active
        	ori	R16,0b00000001
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack
;*******************************************************

SET_DEF:
		clr	R16
		STD	Y+ParamErr-RAMSTRT,R16 
		rjmp	ACKAckt


;*******************************************************
KEY_TYPE:	ldi	ZL,Low(Ktype1)
		ldi	ZH,High(Ktype1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001

		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 Ktype1:	rcall	Read_Data
		brcc	Ktype2
		ret
 Ktype2:	mov	R16,R1				;parameter byte


	;ide jon az feldolgozas!!

 		LDD	R16,Y+PS2Flag-RAMSTRT
		andi	R16,0b11110111			;parameter renben megjott, torolve a bit
		ori	R16,0b00000001			;ACK kuldes kell
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack

;*******************************************************
KEY_BREAK:	ldi	ZL,Low(Kbreak1)
		ldi	ZH,High(Kbreak1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 Kbreak1:	rcall	Read_Data
		brcc	Kbreak2
		ret
 Kbreak2:	mov	R16,R1				;parameter byte


	;ide jon a feldolgozas!!

 		LDD	R16,Y+PS2Flag-RAMSTRT
		andi	R16,0b11110111			;parameter renben megjott, torolve a bit
		ori	R16,0b00000001			;ACK kuldes kell
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack

;*******************************************************
KEY_MAKE:	ldi	ZL,Low(Kmake1)
		ldi	ZH,High(Kmake1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK & Parameter bit flag active
		ori	R16,0b00001001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 Kmake1:	rcall	Read_Data
		brcc	Kmake2
		ret
 Kmake2:	mov	R16,R1				;parameter byte


	;ide jon a feldolgozas!!

 		LDD	R16,Y+PS2Flag-RAMSTRT
		andi	R16,0b11110111			;parameter renben megjott, torolve a bit
		ori	R16,0b00000001			;ACK kuldes kell
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack


;*******************************************************
RESEND:		LDD	R16,Y+PS2Flag-RAMSTRT		;Resend last data!
		ori	R16,0b00000010
		STD	Y+PS2Flag-RAMSTRT,R16
		rjmp	EndBack
		
;*******************************************************
;* Keyboard RESET
;*******************************************************
KRES:		ldi	ZL,Low(KRES1)
		ldi	ZH,High(KRES1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK flag active
		ori	R16,0b00000001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret


 KRES1:		ldi	R16,0xAA			;Basic Assurance Test Result	
		mov	R1,R16
		rcall	SendByte			;Send Result of BAT
		clr	R16
		STD	Y+ParamErr-RAMSTRT,R16 
		ldi	R16,0
		STD	Y+PS2Flag-RAMSTRT,R16 		;Calears Flags
		rjmp	EndBack


;*******************************************************
READID:		ldi	ZL,Low(READID1)
		ldi	ZH,High(READID1)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
		LDD	R16,Y+PS2Flag-RAMSTRT		;Make ACK flag active
		ori	R16,0b00000001
		STD	Y+PS2Flag-RAMSTRT,R16
 		ret
 READID1:	ldi	ZL,Low(READID2)
		ldi	ZH,High(READID2)
		STD	Y+PS2NextA+0-RAMSTRT,ZL
		STD	Y+PS2NextA+1-RAMSTRT,ZH
              	ldi	R16,0xD0			;Eredeti: 0xAB
		mov	R1,R16
		rcall	SendByte			;Send Result of BAT
		ret
 READID2: 	ldi	R16,0xDE			;Eredeti: 0x83
		mov	R1,R16
		rcall	SendByte
EndBack:      	ldi	R16,0
		STD	Y+PS2NextA+0-RAMSTRT,R16
		STD	Y+PS2NextA+1-RAMSTRT,R16
		ret
;*******************************************************
